@@ -14,6 +14,8 @@
 #' @param brace.newline whether to put the left brace \code{\{} to a new line
 #'   (default \code{FALSE})
 #' @param indent number of spaces to indent the code (default 4)
+#' @param wrap whether to wrap comments to the linewidth determined by
+#'   \code{width.cutoff} (note that roxygen comments will never be wrapped)
 #' @param output output to the console or a file using \code{\link{cat}}?
 #' @param text an alternative way to specify the input: if it is \code{NULL},
 #'   the function will read the source code from the \code{source} argument;
@@ -33,8 +35,8 @@
 #' @author Yihui Xie <\url{https://yihui.name}> with substantial contribution
 #'   from Yixuan Qiu <\url{http://yixuan.cos.name}>
 #' @seealso \code{\link{parse}}, \code{\link{deparse}}
-#' @references \url{https://yihui.name/formatR} (an introduction to this package,
-#'   with examples and further notes)
+#' @references \url{https://yihui.name/formatR} (an introduction to this
+#'   package, with examples and further notes)
 #' @import utils
 #' @export
 #' @example inst/examples/tidy.source.R
@@ -44,6 +46,7 @@ tidy_source = function(
   arrow = getOption('formatR.arrow', FALSE),
   brace.newline = getOption('formatR.brace.newline', FALSE),
   indent = getOption('formatR.indent', 4),
+  wrap = getOption('formatR.wrap', TRUE),
   output = TRUE, text = NULL,
   width.cutoff = getOption('width'), ...
 ) {
@@ -51,10 +54,9 @@ tidy_source = function(
     if (source == 'clipboard' && Sys.info()['sysname'] == 'Darwin') {
       source = pipe('pbpaste')
     }
-  } else {
-    source = textConnection(text); on.exit(close(source), add = TRUE)
+    text = readLines(source, warn = FALSE)
   }
-  text = readLines(source, warn = FALSE)
+  enc = special_encoding(text)
   if (length(text) == 0L || all(grepl('^\\s*$', text))) {
     if (output) cat('\n', ...)
     return(list(text.tidy = text, text.mask = text))
@@ -65,7 +67,7 @@ tidy_source = function(
     n2 = attr(regexpr('\n*$', one), 'match.length')
   }
   on.exit(.env$line_break <- NULL, add = TRUE)
-  if (comment) text = mask_comments(text, width.cutoff, blank)
+  if (comment) text = mask_comments(text, width.cutoff, blank, wrap)
   text.mask = tidy_block(text, width.cutoff, arrow && length(grep('=', text)))
   text.tidy = if (comment) unmask_source(text.mask) else text.mask
   text.tidy = reindent_lines(text.tidy, indent)
@@ -73,7 +75,10 @@ tidy_source = function(
   # restore new lines in the beginning and end
   if (blank) text.tidy = c(rep('', n1), text.tidy, rep('', n2))
   if (output) cat(text.tidy, sep = '\n', ...)
-  invisible(list(text.tidy = text.tidy, text.mask = text.mask))
+  invisible(list(
+    text.tidy = restore_encoding(text.tidy, enc),
+    text.mask = restore_encoding(text.mask, enc)
+  ))
 }
 
 ## if you have variable names like this in your code, then you really beat me...
